/**
 * EasyBeans
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansCXFInvoker.java 2546 2008-02-29 13:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.cxf;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Exchange;
import org.apache.cxf.message.FaultMode;
import org.apache.cxf.service.invoker.AbstractInvoker;
import org.ow2.easybeans.api.bean.EasyBeansSLSB;
import org.ow2.easybeans.container.session.stateless.StatelessSessionFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.api.Pool;

/**
 * EasyBeans invoker that will take an instance from the pool, call the method
 * and then put it back in the pool.
 * @author Florent Benoit
 */
public class EasyBeansCXFInvoker extends AbstractInvoker {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EasyBeansCXFInvoker.class);

    /**
     * The factory that will be used for obtaining an instance.
     */
    private StatelessSessionFactory factory = null;

    /**
     * Build a new invoker for the given stateless factory.
     * @param factory the stateless factory
     */
    public EasyBeansCXFInvoker(final StatelessSessionFactory factory) {
        this.factory = factory;
    }

    /**
     * Creates and returns a service object depending on the scope. For now, we
     * return null as me managed our own instance.
     * @param context the exchange object.
     * @return a given service object.
     */
    @Override
    public Object getServiceObject(final Exchange context) {
        // Do nothing
        return null;
    }

    /**
     * Invoke the EJB3 bean with the given parameters.
     * @param exchange some parameters
     * @param serviceObject is null as we will use our pool
     * @param m the method that sould be use for the invocation
     * @param params the parameters of the method
     * @return the result of the invocation.
     */
    @Override
    protected Object invoke(final Exchange exchange, final Object serviceObject, final Method m, final List<Object> params) {
        logger.debug("Invoker : Exchange = {0}, serviceObject = {1}, method = {2} , params = {3}", exchange, serviceObject, m,
                params);

        Object res;
        try {
            Object[] paramArray = new Object[] {};
            if (params != null) {
                paramArray = params.toArray();
            }

            paramArray = insertExchange(m, paramArray, exchange);

            // Call the EJB
            Pool<EasyBeansSLSB, Long> pool = this.factory.getPool();
            EasyBeansSLSB bean = pool.get();
            try {
                res = m.invoke(bean, paramArray);
            } finally {
                pool.release(bean);
            }

            if (exchange.isOneWay()) {
                return null;
            }

            List<Object> retList = new ArrayList<Object>();
            if (!((Class) m.getReturnType()).getName().equals("void")) {
                retList.add(res);
            }
            return retList;
        } catch (InvocationTargetException e) {
            Throwable t = e.getCause();
            if (t == null) {
                t = e;
            }
            exchange.getInMessage().put(FaultMode.class, FaultMode.CHECKED_APPLICATION_FAULT);
            throw new Fault(t);
        } catch (Exception e) {
            exchange.getInMessage().put(FaultMode.class, FaultMode.UNCHECKED_APPLICATION_FAULT);
            throw new Fault(e);
        }
    }

}
